@@ -24,6 +24,13 @@ QString TagLibExtractorTest::testFilePath(const QString& fileName) const
     return QLatin1String(INDEXER_TESTS_SAMPLE_FILES_PATH) + QLatin1Char('/') + fileName;
 }
 
+void TagLibExtractorTest::initTestCase()
+{
+    QFile imgFile(testFilePath("cover.jpg"));
+    imgFile.open(QIODevice::ReadOnly);
+    m_coverImage = imgFile.readAll();
+}
+
 const QStringList TagLibExtractorTest::propertyEnumNames(const QList<KFileMetaData::Property::Property>& keys) const
 {
     QStringList result;
@@ -53,7 +60,7 @@ void TagLibExtractorTest::testPropertyTypes()
     plugin.extract(&resultOpus);
 
     auto testForType = [](SimpleExtractionResult &result, Property::Property prop) {
-            QCOMPARE(result.properties().value(prop).type(), PropertyInfo(prop).valueType());
+            QCOMPARE(result.properties().value(prop).userType(), PropertyInfo(prop).valueType());
     };
 
     QCOMPARE(resultOpus.types().size(), 1);
@@ -235,8 +242,8 @@ void TagLibExtractorTest::testVorbisCommentMultivalue()
     SimpleExtractionResult result(testFilePath(fileName), mimeType);
     plugin.extract(&result);
 
-    QCOMPARE(result.properties(KFileMetaData::MultiMap).values(Property::Artist), QVariantList({QStringLiteral("Artist1"), QStringLiteral("Artist2")}));
-    QCOMPARE(result.properties(KFileMetaData::MultiMap).values(Property::Genre), QVariantList({QStringLiteral("Genre1"), QStringLiteral("Genre2")}));
+    QCOMPARE(result.properties().values(Property::Artist), QVariantList({QStringLiteral("Artist1"), QStringLiteral("Artist2")}));
+    QCOMPARE(result.properties().values(Property::Genre), QVariantList({QStringLiteral("Genre1"), QStringLiteral("Genre2")}));
 }
 
 void TagLibExtractorTest::testVorbisCommentMultivalue_data()
@@ -610,7 +617,7 @@ void  TagLibExtractorTest::testNoMetadata()
     SimpleExtractionResult extracted(path, mimeType);
     plugin.extract(&extracted);
 
-    const auto resultList = extracted.properties(KFileMetaData::MultiMap);
+    const auto resultList = extracted.properties();
     const auto resultKeys = resultList.uniqueKeys();
 
     const QSet<KFileMetaData::Property::Property> resultKeySet(resultKeys.begin(), resultKeys.end());
@@ -623,14 +630,14 @@ void  TagLibExtractorTest::testNoMetadata()
         const auto propNames =  propertyEnumNames(excessKeys.values()).join(QLatin1String(", "));
         if (failMessage.isEmpty()) {
             const auto message = QStringLiteral("Excess properties: %1").arg(propNames);
-            QWARN(qPrintable(message));
+            qWarning() << message;
         } else {
             QEXPECT_FAIL("", qPrintable(QStringLiteral("%1: %2").arg(failMessage).arg(propNames)), Continue);
         }
     } else if (!missingKeys.isEmpty()) {
         const auto message = QStringLiteral("Missing properties: %1")
                 .arg(propertyEnumNames(missingKeys.values()).join(QLatin1String(", ")));
-        QWARN(qPrintable(message));
+        qWarning() << message;
     }
     QCOMPARE(resultKeys, expectedKeys);
     if (!failMessage.isEmpty()) {
@@ -662,4 +669,77 @@ void TagLibExtractorTest::testRobustness()
     plugin.extract(&extracted);
 }
 
+void TagLibExtractorTest::testImageData()
+{
+    QFETCH(QString, fileName);
+    QString testAudioFile;
+    TagLibExtractor plugin{this};;
+
+    testAudioFile = testFilePath(fileName);
+    const QString mimeType = mimeDb.mimeTypeForFile(testAudioFile).name();
+    
+    QVERIFY2(!mimeType.isEmpty(), "Failed to determine mimetype");
+    QVERIFY2(plugin.mimetypes().contains(mimeType), qPrintable(mimeType + " not supported by taglib"));
+    
+    SimpleExtractionResult result(testAudioFile, mimeType, ExtractionResult::ExtractImageData);
+    plugin.extract(&result);
+    
+    QCOMPARE(result.imageData().value(EmbeddedImageData::FrontCover), m_coverImage);
+}
+
+void TagLibExtractorTest::testImageData_data()
+{
+    QTest::addColumn<QString>("fileName");
+
+    QTest::addRow("aiff")
+            << QStringLiteral("test.aif")
+            ;
+
+    QTest::addRow("ape")
+            << QStringLiteral("test.ape")
+            ;
+
+    QTest::addRow("opus")
+            << QStringLiteral("test.opus")
+            ;
+
+    QTest::addRow("ogg")
+            << QStringLiteral("test.ogg")
+            ;
+
+    QTest::addRow("flac")
+            << QStringLiteral("test.flac")
+            ;
+
+    QTest::addRow("mp3")
+            << QStringLiteral("test.mp3")
+            ;
+
+    QTest::addRow("m4a")
+            << QStringLiteral("test.m4a")
+            ;
+
+    QTest::addRow("mpc")
+            << QStringLiteral("test.mpc")
+            ;
+
+    QTest::addRow("speex")
+            << QStringLiteral("test.spx")
+            ;
+
+    QTest::addRow("wav")
+            << QStringLiteral("test.wav")
+            ;
+
+    QTest::addRow("wavpack")
+            << QStringLiteral("test.wv")
+            ;
+
+    QTest::addRow("wma")
+            << QStringLiteral("test.wma")
+            ;
+}
+
 QTEST_GUILESS_MAIN(TagLibExtractorTest)
+
+#include "moc_taglibextractortest.cpp"
