@@ -1,43 +1,29 @@
 /*
- * Copyright (C) 2012  Vishesh Handa <me@vhanda.in>
- * Copyright (C) 2016  Varun Joshi <varunj.1011@gmail.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    SPDX-FileCopyrightText: 2012 Vishesh Handa <me@vhanda.in>
+    SPDX-FileCopyrightText: 2016 Varun Joshi <varunj.1011@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include "extractorcollection.h"
-#include "extractor.h"
 #include "extractor_p.h"
 #include "extractorplugin.h"
 #include "externalextractor.h"
 #include "kfilemetadata_debug.h"
 #include "config-kfilemetadata.h"
 
-#include <QMimeDatabase>
+#include <KPluginMetaData>
 #include <QCoreApplication>
-#include <QPluginLoader>
 #include <QDir>
+#include <QMimeDatabase>
 #include <vector>
 
 using namespace KFileMetaData;
 
-class Q_DECL_HIDDEN ExtractorCollection::Private
+class KFileMetaData::ExtractorCollectionPrivate
 {
 public:
-    QHash<QString, Extractor*> m_mimeExtractors;
+    QMultiHash<QString, Extractor*> m_mimeExtractors;
 
     std::vector<Extractor> m_allExtractors;
 
@@ -46,15 +32,12 @@ public:
 };
 
 ExtractorCollection::ExtractorCollection()
-    : d(new Private)
+    : d(new ExtractorCollectionPrivate)
 {
     d->findExtractors();
 }
 
-ExtractorCollection::~ExtractorCollection()
-{
-    delete d;
-}
+ExtractorCollection::~ExtractorCollection() = default;
 
 
 QList<Extractor*> ExtractorCollection::allExtractors()
@@ -68,63 +51,35 @@ QList<Extractor*> ExtractorCollection::allExtractors()
     return plugins;
 }
 
-void ExtractorCollection::Private::findExtractors()
+void ExtractorCollectionPrivate::findExtractors()
 {
-    QStringList plugins;
-    QStringList externalPlugins;
+    const QVector<KPluginMetaData> kfilemetadataPlugins =
+        KPluginMetaData::findPlugins(QStringLiteral("kf6/kfilemetadata"), {}, KPluginMetaData::AllowEmptyMetaData);
+    for (const KPluginMetaData &plugin : kfilemetadataPlugins) {
+        Extractor extractor;
+        extractor.d->m_pluginPath = plugin.fileName();
+        extractor.setAutoDeletePlugin(Extractor::DoNotDeletePlugin);
 
-    const QStringList paths = QCoreApplication::libraryPaths();
-    for (const QString& libraryPath : paths) {
-        QString path(libraryPath + QStringLiteral("/kf5/kfilemetadata"));
-        QDir dir(path);
-        qCDebug(KFILEMETADATA_LOG) << "Searching for extractors:" << dir.path();
-
-        if (!dir.exists()) {
-            continue;
-        }
-
-        const QStringList entryList = dir.entryList(QDir::Files | QDir::NoDotAndDotDot);
-        for (const QString& fileName : entryList) {
-            // Make sure the same plugin is not loaded twice, even if it is
-            // installed in two different locations
-            if (plugins.contains(fileName)) {
-                qCDebug(KFILEMETADATA_LOG) << "Skipping duplicate - " << path << ":" << fileName;
-                continue;
-            }
-
-            plugins << fileName;
-
-            Extractor extractor;
-            extractor.setAutoDeletePlugin(Extractor::DoNotDeletePlugin);
-            auto pluginPath = dir.absoluteFilePath(fileName);
-
-            QPluginLoader loader(pluginPath);
-            auto metadata = loader.metaData().value(QLatin1String("MetaData"));
-            if (metadata.type() == QJsonValue::Object) {
-                qCDebug(KFILEMETADATA_LOG) << "Found plugin with metadata:" << metadata.toObject();
-                auto pluginProperties = metadata.toObject().toVariantMap();
-                extractor.setMetaData(pluginProperties);
-                extractor.d->m_pluginPath = pluginPath;
+        if (!plugin.rawData().isEmpty()) {
+            qCDebug(KFILEMETADATA_LOG) << "Found plugin with metadata:" << extractor.d->m_pluginPath;
+            extractor.setMetaData(plugin.rawData().toVariantMap());
+            m_allExtractors.push_back(std::move(extractor));
+        } else {
+            qCDebug(KFILEMETADATA_LOG) << "Found plugin without metadata:" << extractor.d->m_pluginPath;
+            if (extractor.d->initPlugin() && !extractor.mimetypes().isEmpty()) {
                 m_allExtractors.push_back(std::move(extractor));
-            } else {
-                qCDebug(KFILEMETADATA_LOG) << "Found plugin without metadata:" << pluginPath;
-                extractor.d->m_pluginPath = pluginPath;
-                if (extractor.d->initPlugin() && !extractor.mimetypes().isEmpty()) {
-                    m_allExtractors.push_back(std::move(extractor));
-                }
             }
         }
     }
-    plugins.clear();
 
-    QDir externalPluginDir(QStringLiteral(LIBEXEC_INSTALL_DIR) + QStringLiteral("/kfilemetadata/externalextractors"));
+    QStringList externalPlugins;
+    const QDir externalPluginDir(QStringLiteral(LIBEXEC_INSTALL_DIR "/kfilemetadata/externalextractors"));
     qCDebug(KFILEMETADATA_LOG) << "Searching for external extractors:" << externalPluginDir.path();
     // For external plugins, we look into the directories
     const QStringList externalPluginEntryList = externalPluginDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
-    for (const QString& externalPlugin : externalPluginEntryList) {
+    for (const QString &externalPlugin : externalPluginEntryList) {
         if (externalPlugins.contains(externalPlugin)) {
-            qCDebug(KFILEMETADATA_LOG) << "Skipping duplicate - "
-                << externalPluginDir.path() << ":" << externalPlugin;
+            qCDebug(KFILEMETADATA_LOG) << "Skipping duplicate - " << externalPluginDir.path() << ":" << externalPlugin;
             continue;
         }
 
@@ -133,12 +88,11 @@ void ExtractorCollection::Private::findExtractors()
 
         Extractor extractor;
         auto pluginPath = externalPluginDir.absoluteFilePath(externalPlugin);
-
         ExternalExtractor *plugin = new ExternalExtractor(pluginPath);
         if (plugin && !plugin->mimetypes().isEmpty()) {
-            extractor.setExtractorPlugin(plugin);
-            extractor.setAutoDeletePlugin(Extractor::AutoDeletePlugin);
-            m_allExtractors.push_back(std::move(extractor));
+              extractor.setExtractorPlugin(plugin);
+              extractor.setAutoDeletePlugin(Extractor::AutoDeletePlugin);
+              m_allExtractors.push_back(std::move(extractor));
         }
     }
     externalPlugins.clear();
@@ -149,18 +103,18 @@ void ExtractorCollection::Private::findExtractors()
             auto mimetypeProperties = pluginProperties[QLatin1String("MimeTypes")];
             const auto mimetypes = mimetypeProperties.toMap().keys();
             for (const QString &mimetype : mimetypes) {
-                m_mimeExtractors.insertMulti(mimetype, &extractor);
+                m_mimeExtractors.insert(mimetype, &extractor);
             }
         } else if (extractor.d->m_plugin) {
             const auto mimetypes = extractor.mimetypes();
             for (const QString &mimetype : mimetypes) {
-                m_mimeExtractors.insertMulti(mimetype, &extractor);
+                m_mimeExtractors.insert(mimetype, &extractor);
             }
         }
     }
 }
 
-QList<Extractor*> ExtractorCollection::Private::getExtractors(const QString& mimetype)
+QList<Extractor*> ExtractorCollectionPrivate::getExtractors(const QString& mimetype)
 {
     QList<Extractor*> extractors = m_mimeExtractors.values(mimetype);
 
