@@ -1,20 +1,7 @@
 /*
-    <one line to give the library's name and an idea of what it does.>
-    Copyright (C) 2012  Vishesh Handa <me@vhanda.in>
+    SPDX-FileCopyrightText: 2012 Vishesh Handa <me@vhanda.in>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 
@@ -27,6 +14,9 @@ using namespace KFileMetaData;
 Exiv2Extractor::Exiv2Extractor(QObject* parent)
     : ExtractorPlugin(parent)
 {
+#ifdef EXV_ENABLE_BMFF
+    Exiv2::enableBMFF(true);
+#endif
 }
 
 namespace
@@ -39,8 +29,12 @@ static const QStringList supportedMimeTypes = {
     QStringLiteral("image/pgf"),
     QStringLiteral("image/png"),
     QStringLiteral("image/tiff"),
-#ifdef HAVE_WEBP_SUPPORT
     QStringLiteral("image/webp"),
+#ifdef EXV_ENABLE_BMFF
+    QStringLiteral("image/avif"),
+    QStringLiteral("image/heif"),
+    QStringLiteral("image/jxl"),
+    QStringLiteral("image/x-canon-cr3"),
 #endif
     QStringLiteral("image/x-exv"),
     QStringLiteral("image/x-canon-cr2"),
@@ -68,7 +62,7 @@ QVariant toVariantDateTime(const Exiv2::Value& value)
         QDateTime val = ExtractorPlugin::dateTimeFromString(QString::fromLatin1(value.toString().c_str()));
         if (val.isValid()) {
             // Datetime is stored in exif as local time.
-            val.setUtcOffset(0);
+            val.setOffsetFromUtc(0);
             return QVariant(val);
         }
     }
@@ -79,15 +73,20 @@ QVariant toVariantDateTime(const Exiv2::Value& value)
 QVariant toVariantLong(const Exiv2::Value& value)
 {
     if (value.typeId() == Exiv2::unsignedLong || value.typeId() == Exiv2::signedLong) {
+#if EXIV2_TEST_VERSION(0,28,0)
+        qlonglong val = value.toInt64();
+#else
         qlonglong val = value.toLong();
+#endif
         return QVariant(val);
     }
 
     QString str(toString(value));
     bool ok = false;
     int val = str.toInt(&ok);
-    if (ok)
+    if (ok) {
         return QVariant(val);
+    }
 
     return QVariant();
 }
@@ -102,8 +101,9 @@ QVariant toVariantDouble(const Exiv2::Value& value)
     QString str(toString(value));
     bool ok = false;
     double val = str.toDouble(&ok);
-    if (ok)
+    if (ok) {
         return QVariant(val);
+    }
 
     return QVariant();
 }
@@ -111,27 +111,28 @@ QVariant toVariantDouble(const Exiv2::Value& value)
 QVariant toVariantString(const Exiv2::Value& value)
 {
     QString str = toString(value);
-    if (!str.isEmpty())
+    if (!str.isEmpty()) {
         return QVariant(str);
+    }
 
     return QVariant();
 }
 
-QVariant toVariant(const Exiv2::Value& value, QVariant::Type type) {
+QVariant toVariant(const Exiv2::Value& value, QMetaType::Type type) {
     if (value.count() == 0) {
         return QVariant();
     }
     switch (type) {
-    case QVariant::Int:
+    case QMetaType::Int:
         return toVariantLong(value);
 
-    case QVariant::DateTime:
+    case QMetaType::QDateTime:
         return toVariantDateTime(value);
 
-    case QVariant::Double:
+    case QMetaType::Double:
         return toVariantDouble(value);
 
-    case QVariant::String:
+    case QMetaType::QString:
     default:
         return toVariantString(value);
     }
@@ -169,6 +170,10 @@ void Exiv2Extractor::extract(ExtractionResult* result)
     }
     result->addType(Type::Image);
 
+    if (!(result->inputFlags() & ExtractionResult::ExtractMetaData)) {
+        return;
+    }
+
     if (image->pixelHeight()) {
         result->add(Property::Height, image->pixelHeight());
     }
@@ -184,41 +189,43 @@ void Exiv2Extractor::extract(ExtractionResult* result)
 
     const Exiv2::ExifData& data = image->exifData();
 
-    add(result, data, Property::Manufacturer, "Exif.Image.Make", QVariant::String);
-    add(result, data, Property::Model, "Exif.Image.Model", QVariant::String);
-    add(result, data, Property::Description, "Exif.Image.ImageDescription", QVariant::String);
-    add(result, data, Property::Artist, "Exif.Image.Artist", QVariant::String);
-    add(result, data, Property::Copyright, "Exif.Image.Copyright", QVariant::String);
-    add(result, data, Property::Generator, "Exif.Image.Software", QVariant::String);
-    add(result, data, Property::ImageDateTime, "Exif.Image.DateTime", QVariant::DateTime);
-    add(result, data, Property::ImageOrientation, "Exif.Image.Orientation", QVariant::Int);
-    add(result, data, Property::PhotoFlash, "Exif.Photo.Flash", QVariant::Int);
-    add(result, data, Property::PhotoPixelXDimension, "Exif.Photo.PixelXDimension", QVariant::Int);
-    add(result, data, Property::PhotoPixelYDimension, "Exif.Photo.PixelYDimension", QVariant::Int);
-    add(result, data, Property::PhotoDateTimeOriginal, "Exif.Photo.DateTimeOriginal", QVariant::DateTime);
-    add(result, data, Property::PhotoFocalLength, "Exif.Photo.FocalLength", QVariant::Double);
-    add(result, data, Property::PhotoFocalLengthIn35mmFilm, "Exif.Photo.FocalLengthIn35mmFilm", QVariant::Double);
-    add(result, data, Property::PhotoExposureTime, "Exif.Photo.ExposureTime", QVariant::Double);
-    add(result, data, Property::PhotoExposureBiasValue, "Exif.Photo.ExposureBiasValue", QVariant::Double);
-    add(result, data, Property::PhotoFNumber, "Exif.Photo.FNumber", QVariant::Double);
-    add(result, data, Property::PhotoApertureValue, "Exif.Photo.ApertureValue", QVariant::Double);
-    add(result, data, Property::PhotoWhiteBalance, "Exif.Photo.WhiteBalance", QVariant::Int);
-    add(result, data, Property::PhotoMeteringMode, "Exif.Photo.MeteringMode", QVariant::Int);
-    add(result, data, Property::PhotoISOSpeedRatings, "Exif.Photo.ISOSpeedRatings", QVariant::Int);
-    add(result, data, Property::PhotoSaturation, "Exif.Photo.Saturation", QVariant::Int);
-    add(result, data, Property::PhotoSharpness, "Exif.Photo.Sharpness", QVariant::Int);
+    add(result, data, Property::Manufacturer, "Exif.Image.Make", QMetaType::QString);
+    add(result, data, Property::Model, "Exif.Image.Model", QMetaType::QString);
+    add(result, data, Property::Description, "Exif.Image.ImageDescription", QMetaType::QString);
+    add(result, data, Property::Artist, "Exif.Image.Artist", QMetaType::QString);
+    add(result, data, Property::Copyright, "Exif.Image.Copyright", QMetaType::QString);
+    add(result, data, Property::Generator, "Exif.Image.Software", QMetaType::QString);
+    add(result, data, Property::ImageDateTime, "Exif.Image.DateTime", QMetaType::QDateTime);
+    add(result, data, Property::ImageOrientation, "Exif.Image.Orientation", QMetaType::Int);
+    add(result, data, Property::PhotoFlash, "Exif.Photo.Flash", QMetaType::Int);
+    add(result, data, Property::PhotoPixelXDimension, "Exif.Photo.PixelXDimension", QMetaType::Int);
+    add(result, data, Property::PhotoPixelYDimension, "Exif.Photo.PixelYDimension", QMetaType::Int);
+    add(result, data, Property::PhotoDateTimeOriginal, "Exif.Photo.DateTimeOriginal", QMetaType::QDateTime);
+    add(result, data, Property::PhotoFocalLength, "Exif.Photo.FocalLength", QMetaType::Double);
+    add(result, data, Property::PhotoFocalLengthIn35mmFilm, "Exif.Photo.FocalLengthIn35mmFilm", QMetaType::Double);
+    add(result, data, Property::PhotoExposureTime, "Exif.Photo.ExposureTime", QMetaType::Double);
+    add(result, data, Property::PhotoExposureBiasValue, "Exif.Photo.ExposureBiasValue", QMetaType::Double);
+    add(result, data, Property::PhotoFNumber, "Exif.Photo.FNumber", QMetaType::Double);
+    add(result, data, Property::PhotoApertureValue, "Exif.Photo.ApertureValue", QMetaType::Double);
+    add(result, data, Property::PhotoWhiteBalance, "Exif.Photo.WhiteBalance", QMetaType::Int);
+    add(result, data, Property::PhotoMeteringMode, "Exif.Photo.MeteringMode", QMetaType::Int);
+    add(result, data, Property::PhotoISOSpeedRatings, "Exif.Photo.ISOSpeedRatings", QMetaType::Int);
+    add(result, data, Property::PhotoSaturation, "Exif.Photo.Saturation", QMetaType::Int);
+    add(result, data, Property::PhotoSharpness, "Exif.Photo.Sharpness", QMetaType::Int);
 
     double latitude = fetchGpsDouble(data, "Exif.GPSInfo.GPSLatitude");
     double longitude = fetchGpsDouble(data, "Exif.GPSInfo.GPSLongitude");
     double altitude = fetchGpsAltitude(data);
 
     QByteArray latRef = fetchByteArray(data, "Exif.GPSInfo.GPSLatitudeRef");
-    if (!latRef.isEmpty() && latRef[0] == 'S')
+    if (!latRef.isEmpty() && latRef[0] == 'S') {
         latitude *= -1;
+    }
 
     QByteArray longRef = fetchByteArray(data, "Exif.GPSInfo.GPSLongitudeRef");
-    if (!longRef.isEmpty() && longRef[0] == 'W')
+    if (!longRef.isEmpty() && longRef[0] == 'W') {
         longitude *= -1;
+    }
 
     if (!std::isnan(latitude)) {
         result->add(Property::PhotoGpsLatitude, latitude);
@@ -235,13 +242,14 @@ void Exiv2Extractor::extract(ExtractionResult* result)
 
 void Exiv2Extractor::add(ExtractionResult* result, const Exiv2::ExifData& data,
                          Property::Property prop, const char* name,
-                         QVariant::Type type)
+                         QMetaType::Type type)
 {
     Exiv2::ExifData::const_iterator it = data.findKey(Exiv2::ExifKey(name));
     if (it != data.end()) {
         QVariant value = toVariant(it->value(), type);
-        if (!value.isNull())
+        if (!value.isNull()) {
             result->add(prop, value);
+        }
     }
 }
 
@@ -304,7 +312,11 @@ double Exiv2Extractor::fetchGpsAltitude(const Exiv2::ExifData& data)
         it = data.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSAltitudeRef"));
         if (it != data.end() && it->count() > 0 &&
             (it->value().typeId() == Exiv2::unsignedByte || it->value().typeId() == Exiv2::signedByte)) {
+#if EXIV2_TEST_VERSION(0,28,0)
+            auto altRef = it->value().toInt64();
+#else
             auto altRef = it->value().toLong();
+#endif
             if (altRef) {
                 alt = -1.0 * ratio.first / ratio.second;
             } else {
@@ -325,3 +337,5 @@ QByteArray Exiv2Extractor::fetchByteArray(const Exiv2::ExifData& data, const cha
 
     return QByteArray();
 }
+
+#include "moc_exiv2extractor.cpp"
