@@ -1,24 +1,12 @@
 /*
-    <one line to give the library's name and an idea of what it does.>
-    Copyright (C) 2012  Vishesh Handa <me@vhanda.in>
+    SPDX-FileCopyrightText: 2012 Vishesh Handa <me@vhanda.in>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 
 #include "taglibextractor.h"
+#include "embeddedimagedata.h"
 #include "kfilemetadata_debug.h"
 
 // Taglib includes
@@ -28,6 +16,7 @@
 #include <tpropertymap.h>
 #include <aifffile.h>
 #include <apefile.h>
+#include <apetag.h>
 #include <asffile.h>
 #include <flacfile.h>
 #include <mp4file.h>
@@ -41,16 +30,14 @@
 #include <wavpackfile.h>
 #include <asftag.h>
 #include <asfattribute.h>
+#include <asfpicture.h>
 #include <id3v2tag.h>
 #include <mp4tag.h>
 #include <popularimeterframe.h>
 
 using namespace KFileMetaData;
 
-TagLibExtractor::TagLibExtractor(QObject* parent)
-    : ExtractorPlugin(parent)
-{
-}
+namespace {
 
 const QStringList supportedMimeTypes = {
     QStringLiteral("audio/flac"),
@@ -59,8 +46,9 @@ const QStringList supportedMimeTypes = {
     QStringLiteral("audio/mpeg3"),
     QStringLiteral("audio/ogg"),
     QStringLiteral("audio/opus"),
-    QStringLiteral("audio/speex"),
     QStringLiteral("audio/wav"),
+    QStringLiteral("audio/vnd.audible.aax"),
+    QStringLiteral("audio/vnd.wave"),
     QStringLiteral("audio/x-aiff"),
     QStringLiteral("audio/x-aifc"),
     QStringLiteral("audio/x-ape"),
@@ -74,15 +62,10 @@ const QStringList supportedMimeTypes = {
     QStringLiteral("audio/x-wavpack"),
 };
 
-QStringList TagLibExtractor::mimetypes() const
-{
-    return supportedMimeTypes;
-}
-
 void extractAudioProperties(TagLib::File* file, ExtractionResult* result)
 {
     TagLib::AudioProperties* audioProp = file->audioProperties();
-    if (audioProp) {
+    if (audioProp && (result->inputFlags() & ExtractionResult::ExtractMetaData)) {
         if (audioProp->length()) {
             // What about the xml duration?
             result->add(Property::Duration, audioProp->length());
@@ -102,11 +85,12 @@ void extractAudioProperties(TagLib::File* file, ExtractionResult* result)
     }
 }
 
-void TagLibExtractor::readGenericProperties(const TagLib::PropertyMap &savedProperties, ExtractionResult* result)
+void readGenericProperties(const TagLib::PropertyMap &savedProperties, ExtractionResult* result)
 {
-    if (savedProperties.isEmpty()) {
+    if (!(result->inputFlags() & ExtractionResult::ExtractMetaData) || savedProperties.isEmpty()) {
         return;
     }
+
     if (savedProperties.contains("TITLE")) {
         result->add(Property::Title, TStringToQString(savedProperties["TITLE"].toString()).trimmed());
     }
@@ -161,7 +145,7 @@ void TagLibExtractor::readGenericProperties(const TagLib::PropertyMap &savedProp
         result->add(Property::Compilation, TStringToQString(savedProperties["COMPILATION"].toString()).trimmed());
     }
     if (savedProperties.contains("LYRICS")) {
-        result->add(Property::Lyrics, TStringToQString(savedProperties["LYRICS"].toString()).trimmed());
+        result->add(Property::Lyrics, TStringToQString(savedProperties["LYRICS"].toString()).trimmed().replace(QLatin1Char('\r'), QLatin1Char('\n'))); // Convert old Mac line endings
     }
     if (savedProperties.contains("ARTIST")) {
         const auto artists = savedProperties["ARTIST"];
@@ -260,11 +244,12 @@ void TagLibExtractor::readGenericProperties(const TagLib::PropertyMap &savedProp
     }
 }
 
-void TagLibExtractor::extractId3Tags(TagLib::ID3v2::Tag* Id3Tags, ExtractionResult* result)
+void extractId3Tags(TagLib::ID3v2::Tag* Id3Tags, ExtractionResult* result)
 {
-    if (Id3Tags->isEmpty()) {
+    if (!(result->inputFlags() & ExtractionResult::ExtractMetaData) || Id3Tags->isEmpty()) {
         return;
     }
+
     TagLib::ID3v2::FrameList lstID3v2;
 
     /*
@@ -309,27 +294,164 @@ void TagLibExtractor::extractId3Tags(TagLib::ID3v2::Tag* Id3Tags, ExtractionResu
     }
 }
 
-void TagLibExtractor::extractMp4Tags(TagLib::MP4::Tag* mp4Tags, ExtractionResult* result)
+template<typename ImageType>
+EmbeddedImageData::ImageType mapTaglibType(const ImageType type)
 {
-    if (mp4Tags->isEmpty()) {
+    switch (type) {
+        case ImageType::FrontCover:
+            return EmbeddedImageData::FrontCover;
+         case ImageType::Other:
+             return EmbeddedImageData::Other;
+         case ImageType::FileIcon:
+              return EmbeddedImageData::FileIcon;
+         case ImageType::OtherFileIcon:
+              return EmbeddedImageData::OtherFileIcon;
+         case ImageType::BackCover:
+              return EmbeddedImageData::BackCover;
+         case ImageType::LeafletPage:
+              return EmbeddedImageData::LeafletPage;
+         case ImageType::Media:
+              return EmbeddedImageData::Media;
+         case ImageType::LeadArtist:
+              return EmbeddedImageData::LeadArtist;
+         case ImageType::Artist:
+              return EmbeddedImageData::Artist;
+         case ImageType::Conductor:
+              return EmbeddedImageData::Conductor;
+         case ImageType::Band:
+              return EmbeddedImageData::Band;
+         case ImageType::Composer:
+              return EmbeddedImageData::Composer;
+         case ImageType::Lyricist:
+              return EmbeddedImageData::Lyricist;
+         case ImageType::RecordingLocation:
+              return EmbeddedImageData::RecordingLocation;
+         case ImageType::DuringRecording:
+              return EmbeddedImageData::DuringRecording;
+         case ImageType::DuringPerformance:
+              return EmbeddedImageData::DuringPerformance;
+         case ImageType::MovieScreenCapture:
+              return EmbeddedImageData::MovieScreenCapture;
+         case ImageType::ColouredFish:
+              return EmbeddedImageData::ColouredFish;
+         case ImageType::Illustration:
+              return EmbeddedImageData::Illustration;
+         case ImageType::BandLogo:
+              return EmbeddedImageData::BandLogo;
+         case ImageType::PublisherLogo:
+              return EmbeddedImageData::PublisherLogo;
+         default:
+            return EmbeddedImageData::Unknown;
+    }
+}
+
+QMap<EmbeddedImageData::ImageType, QByteArray>
+extractId3Cover(const TagLib::ID3v2::Tag* Id3Tags,
+                const EmbeddedImageData::ImageTypes types)
+{
+    QMap<EmbeddedImageData::ImageType, QByteArray> images;
+    if (!types || Id3Tags->isEmpty()) {
+        return images;
+    }
+
+    // Attached Picture.
+    TagLib::ID3v2::FrameList lstID3v2 = Id3Tags->frameListMap()["APIC"];
+
+    using PictureFrame = TagLib::ID3v2::AttachedPictureFrame;
+    for (const auto& frame : std::as_const(lstID3v2)) {
+        const auto *coverFrame = static_cast<PictureFrame *>(frame);
+        const auto imageType = mapTaglibType<PictureFrame::Type>(coverFrame->type());
+        if (types & imageType) {
+            const auto& picture = coverFrame->picture();
+            images.insert(imageType, QByteArray(picture.data(), picture.size()));
+        }
+    }
+    return images;
+}
+
+QMap<EmbeddedImageData::ImageType, QByteArray>
+extractFlacCover(const TagLib::List<TagLib::FLAC::Picture *> picList,
+                 const EmbeddedImageData::ImageTypes types)
+{
+    QMap<EmbeddedImageData::ImageType, QByteArray> images;
+    if (!types || picList.isEmpty()) {
+        return images;
+    }
+
+    for (const auto& picture : std::as_const(picList)) {
+        const auto imageType = mapTaglibType<TagLib::FLAC::Picture::Type>(picture->type());
+        if (types & imageType) {
+            images.insert(imageType, QByteArray(picture->data().data(), picture->data().size()));
+        }
+    }
+    return images;
+}
+
+void extractMp4Tags(TagLib::MP4::Tag* mp4Tags, ExtractionResult* result)
+{
+    if (!(result->inputFlags() & ExtractionResult::ExtractMetaData) || mp4Tags->isEmpty()) {
         return;
     }
-    TagLib::MP4::ItemListMap allTags = mp4Tags->itemListMap();
+
+    auto ratingItem = mp4Tags->item("rate");
 
     /*
      * There is no standard regarding ratings. Mimic MediaMonkey's behavior
      * with a range of 0 to 100 (stored in steps of 10) and make it compatible
      * with baloo rating with a range from 0 to 10.
      */
-    TagLib::MP4::ItemListMap::Iterator itRating = allTags.find("rate");
-    if (itRating != allTags.end()) {
-        result->add(Property::Rating, itRating->second.toStringList().toString().toInt() / 10);
+    if (ratingItem.isValid()) {
+        result->add(Property::Rating, ratingItem.toStringList().toString().toInt() / 10);
     }
 }
 
-void TagLibExtractor::extractAsfTags(TagLib::ASF::Tag* asfTags, ExtractionResult* result)
+QMap<EmbeddedImageData::ImageType, QByteArray>
+extractMp4Cover(const TagLib::MP4::Tag* mp4Tags,
+                const EmbeddedImageData::ImageTypes types)
 {
-    if (asfTags->isEmpty()) {
+    QMap<EmbeddedImageData::ImageType, QByteArray> images;
+    TagLib::MP4::Item coverArtItem = mp4Tags->item("covr");
+    if (!(types & EmbeddedImageData::FrontCover) || !coverArtItem.isValid()) {
+        return images;
+    }
+
+    const TagLib::MP4::CoverArtList coverArtList = coverArtItem.toCoverArtList();
+    if (!coverArtList.isEmpty()) {
+        const TagLib::MP4::CoverArt& cover = coverArtList.front();
+        images.insert(EmbeddedImageData::FrontCover, QByteArray(cover.data().data(), cover.data().size()));
+    }
+    return images;
+}
+
+QMap<EmbeddedImageData::ImageType, QByteArray>
+extractApeCover(const TagLib::APE::Tag* apeTags,
+                const EmbeddedImageData::ImageTypes types)
+{
+    QMap<EmbeddedImageData::ImageType, QByteArray> images;
+    if (!(types & EmbeddedImageData::FrontCover) || apeTags->isEmpty()) {
+        return images;
+    }
+
+    TagLib::APE::ItemListMap lstApe = apeTags->itemListMap();
+    TagLib::APE::ItemListMap::ConstIterator itApe;
+
+    /* The cover art tag for APEv2 tags starts with the filename as string
+     * with zero termination followed by the actual picture data */
+    itApe = lstApe.find("COVER ART (FRONT)");
+    if (itApe != lstApe.end()) {
+        const auto& picture = (*itApe).second.binaryData();
+        int position = picture.find('\0');
+        if (position >= 0) {
+            position += 1;
+            images.insert(EmbeddedImageData::FrontCover, QByteArray(picture.data() + position, picture.size() - position));
+        }
+    }
+    return images;
+}
+
+void extractAsfTags(TagLib::ASF::Tag* asfTags, ExtractionResult* result)
+{
+    if (!(result->inputFlags() & ExtractionResult::ExtractMetaData) || asfTags->isEmpty()) {
         return;
     }
 
@@ -374,18 +496,62 @@ void TagLibExtractor::extractAsfTags(TagLib::ASF::Tag* asfTags, ExtractionResult
     }
 }
 
+QMap<EmbeddedImageData::ImageType, QByteArray>
+extractAsfCover(const TagLib::ASF::Tag* asfTags,
+                const EmbeddedImageData::ImageTypes types)
+{
+    QMap<EmbeddedImageData::ImageType, QByteArray> images;
+    if (!types || asfTags->isEmpty()) {
+        return images;
+    }
+
+    // Attached Picture.
+    TagLib::ASF::AttributeList lstASF = asfTags->attribute("WM/Picture");
+
+    using Picture = TagLib::ASF::Picture;
+    for (const auto& attribute: std::as_const(lstASF)) {
+        Picture picture = attribute.toPicture();
+        const auto imageType = mapTaglibType<Picture::Type>(picture.type());
+        if (types & imageType) {
+            const auto& pictureData = picture.picture();
+            images.insert(imageType, QByteArray(pictureData.data(), pictureData.size()));
+        }
+    }
+    return images;
+}
+
+} // anonymous namespace
+
+TagLibExtractor::TagLibExtractor(QObject* parent)
+    : ExtractorPlugin(parent)
+{
+}
+
+QStringList TagLibExtractor::mimetypes() const
+{
+    return supportedMimeTypes;
+}
+
 void TagLibExtractor::extract(ExtractionResult* result)
 {
     const QString fileUrl = result->inputUrl();
     const QString mimeType = getSupportedMimeType(result->inputMimetype());
 
     // Open the file readonly. Important if we're sandboxed.
+#if defined Q_OS_WINDOWS
+    TagLib::FileStream stream(fileUrl.toLocal8Bit().constData(), true);
+#else
     TagLib::FileStream stream(fileUrl.toUtf8().constData(), true);
+#endif
     if (!stream.isOpen()) {
         qCWarning(KFILEMETADATA_LOG) << "Unable to open file readonly: " << fileUrl;
         return;
     }
 
+    const EmbeddedImageData::ImageTypes imageTypes{
+        result->inputFlags() & ExtractionResult::ExtractImageData ? EmbeddedImageData::AllImages : 0
+    };
+
     if (mimeType == QLatin1String("audio/mpeg") || mimeType == QLatin1String("audio/mpeg3")
             || mimeType == QLatin1String("audio/x-mpeg")) {
         TagLib::MPEG::File file(&stream, TagLib::ID3v2::FrameFactory::instance(), true);
@@ -393,6 +559,7 @@ void TagLibExtractor::extract(ExtractionResult* result)
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
             if (file.hasID3v2Tag()) {
+                result->addImageData(extractId3Cover(file.ID3v2Tag(), imageTypes));
                 extractId3Tags(file.ID3v2Tag(), result);
             }
         }
@@ -402,15 +569,19 @@ void TagLibExtractor::extract(ExtractionResult* result)
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
             if (file.hasID3v2Tag()) {
+                result->addImageData(extractId3Cover(file.tag(), imageTypes));
                 extractId3Tags(file.tag(), result);
             }
         }
-    } else if (mimeType == QLatin1String("audio/wav") || mimeType == QLatin1String("audio/x-wav")) {
+    } else if (mimeType == QLatin1String("audio/wav") ||
+               mimeType == QLatin1String("audio/vnd.wave") ||
+               mimeType == QLatin1String("audio/x-wav")) {
         TagLib::RIFF::WAV::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
             if (file.hasID3v2Tag()) {
+                result->addImageData(extractId3Cover(file.tag(), imageTypes));
                 extractId3Tags(file.tag(), result);
             }
         }
@@ -419,51 +590,70 @@ void TagLibExtractor::extract(ExtractionResult* result)
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            if (file.APETag()) {
+                result->addImageData(extractApeCover(file.APETag(), imageTypes));
+            }
         }
     } else if (mimeType == QLatin1String("audio/x-ape")) {
         TagLib::APE::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            if (file.APETag()) {
+                result->addImageData(extractApeCover(file.APETag(), imageTypes));
+            }
         }
     } else if (mimeType == QLatin1String("audio/x-wavpack")) {
         TagLib::WavPack::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            if (file.APETag()) {
+                result->addImageData(extractApeCover(file.APETag(), imageTypes));
+            }
         }
-    } else if (mimeType == QLatin1String("audio/mp4")) {
+    } else if ((mimeType == QLatin1String("audio/mp4")) ||
+               (mimeType == QLatin1String("audio/vnd.audible.aax"))) {
         TagLib::MP4::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
             extractMp4Tags(file.tag(), result);
+            result->addImageData(extractMp4Cover(file.tag(), imageTypes));
         }
     } else if (mimeType == QLatin1String("audio/flac")) {
         TagLib::FLAC::File file(&stream, TagLib::ID3v2::FrameFactory::instance(), true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            result->addImageData(extractFlacCover(file.pictureList(), imageTypes));
         }
     } else if (mimeType == QLatin1String("audio/ogg") || mimeType == QLatin1String("audio/x-vorbis+ogg")) {
         TagLib::Ogg::Vorbis::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            if (file.tag()) {
+                result->addImageData(extractFlacCover(file.tag()->pictureList(), imageTypes));
+            }
         }
     } else if (mimeType == QLatin1String("audio/opus") || mimeType == QLatin1String("audio/x-opus+ogg")) {
         TagLib::Ogg::Opus::File file(&stream, true);
         if (file.isValid()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            if (file.tag()) {
+                result->addImageData(extractFlacCover(file.tag()->pictureList(), imageTypes));
+            }
         }
-    } else if (mimeType == QLatin1String("audio/speex") || mimeType == QLatin1String("audio/x-speex+ogg")) {
+    } else if (mimeType == QLatin1String("audio/x-speex+ogg")) {
         TagLib::Ogg::Speex::File file(&stream, true);
         // Workaround for buggy taglib:
         // isValid() returns true for invalid files, but XiphComment* tag() returns a nullptr
         if (file.isValid() && file.tag()) {
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
+            result->addImageData(extractFlacCover(file.tag()->pictureList(), imageTypes));
         }
     } else if (mimeType == QLatin1String("audio/x-ms-wma")) {
         TagLib::ASF::File file(&stream, true);
@@ -471,6 +661,10 @@ void TagLibExtractor::extract(ExtractionResult* result)
             extractAudioProperties(&file, result);
             readGenericProperties(file.properties(), result);
             extractAsfTags(file.tag(), result);
+            TagLib::ASF::Tag* asfTags = dynamic_cast<TagLib::ASF::Tag*>(file.tag());
+            if (asfTags) {
+                result->addImageData(extractAsfCover(asfTags, imageTypes));
+            }
         }
     }
 
@@ -505,3 +699,5 @@ void TagLibExtractor::extract(ExtractionResult* result)
 // Disc number[/total dics]:        TPOS
 // Track number[/total tracks]:     TRCK
 // Genre:                           TCON
+
+#include "moc_taglibextractor.cpp"
